{% extends 'layout.jinja2' %}

{% block subtitle %}Edit {{pagename}} - {% endblock subtitle %}

{% block content %}
<p>
Editing <strong>{{pagename}}</strong>
</p>
<p>You can return to the
<a href="{{request.route_url('view_page', pagename='FrontPage')}}">FrontPage</a>.
</p>
<form action="{{ save_url }}" method="post">
<div class="form-group">
    <textarea class="form-control" name="body" rows="10" cols="60">{{ pagedata }}</textarea>
</div>
<div class="form-group">
    <button type="submit" name="form.submitted" value="Save" class="btn btn-default">Save</button>
</div>
</form>
{% endblock content %}
